from pytracking.utils import TrackerParams, FeatureParams
from pytracking.features.extractor import MultiResolutionExtractor
from pytracking.features import deep
import torch

def parameters():
    params = TrackerParams()

    params.debug = 0
    params.visualization = False

    params.use_gpu = True

    deep_params = TrackerParams()

    params.max_image_sample_size = (14 * 16) ** 2
    params.min_image_sample_size = (14 * 16) ** 2
    params.search_area_scale = 4
    params.feature_size_odd = False

    params.sample_memory_size = 250

    # Conjugate Gradient parameters
    params.CG_iter = 5                     # The number of Conjugate Gradient iterations in each update after the first frame
    params.init_CG_iter = 60            # The total number of Conjugate Gradient iterations used in the first frame
    params.init_GN_iter = 6               # The number of Gauss-Newton iterations used in the first frame (only if the projection matrix is updated)
    params.post_init_CG_iter = 0       # CG iterations to run after GN
    params.fletcher_reeves = False               # Use the Fletcher-Reeves (true) or Polak-Ribiere (false) formula in the Conjugate Gradient
    params.standard_alpha = True        # Use the standard formula for computing the step length in Conjugate Gradient
    params.CG_forgetting_rate = None	 	 	# Forgetting rate of the last conjugate direction

    # Learning parameters
    learning_rate = 0.0075
    deep_params.spatial_training = True
    deep_params.learning_rate = learning_rate
    deep_params.output_sigma_factor = 1/4

    params.scale_factors = torch.ones(1)
    params.train_skipping = 10

    # Spatial filter parameters
    deep_params.kernel_size = (4,4)
    deep_params.filter_reg = 1e-1
    deep_params.projection_reg = 1e-4

    params.window_spatial = False
    params.window_output = True
    params.joint_spatial_filter = False

    # Detection parameters
    params.score_upsample_factor = 1
    params.score_fusion_strategy = 'weightedsum'
    deep_params.translation_weight = 1

    # Init augmentation parameters
    params.augmentation = {'fliplr': True,
                           'rotate': [5, -5, 10, -10, 20, -20, 30, -30, 45,-45, -60, 60],
                           'blur': [(2, 0.2), (0.2, 2), (3,1), (1, 3), (2, 2)],
                           'relativeshift': [(0.6, 0.6), (-0.6, 0.6), (0.6, -0.6), (-0.6,-0.6)],
                           'dropout': (7, 0.2)}

    params.augmentation_expansion_factor = 2
    params.random_shift_factor = 1/3

    deep_params.use_augmentation = True

    # Factorized convolution parameters
    # params.use_projection_matrix = True    # Use projection matrix, i.e. use the factorized convolution formulation
    params.update_projection_matrix = True # Whether the projection matrix should be optimized or not
    params.proj_init_method = 'randn'        # Method for initializing the projection matrix
    # params.spatial_filter_init_method = 'randn'        # Method for initializing the spatial filter
    params.spatial_projection_activation = 'none'
    params.spatial_response_activation = ('mlu', 0.05)
    deep_params.compressed_dim = 64


    # Advanced localization parameters
    params.advanced_localization = True
    params.target_not_found_threshold = -1
    params.distractor_threshold = 100
    params.hard_negative_threshold = 0.3
    params.target_neighborhood_scale = 2.2
    params.dispalcement_scale = 0.7

    params.hard_negative_learning_rate = 0.02
    params.hard_negative_CG_iter = 5
    params.update_scale_when_uncertain = True

    # IoUnet parameters
    params.iounet_augmentation = False
    params.iounet_use_log_scale = True
    params.iounet_k = 3
    params.num_init_random_boxes = 9
    params.box_jitter_pos = 0.1
    params.box_jitter_sz = 0.5
    params.maximal_aspect_ratio = 6
    params.box_refinement_iter = 5
    params.box_refinement_step_length = 1
    params.box_refinement_step_decay = 1

    # Setup the feature extractor (which includes the IoUNet)
    deep_fparams = FeatureParams(feature_params=[deep_params])
    deep_feat = deep.ATOMResNet18(net_path='atom50.pth.tar', output_layers=['layer3'],
                                  fparams=deep_fparams, normalize_power=2)
    params.features = MultiResolutionExtractor([deep_feat])

    params.vot_anno_conversion_type = 'preserve_area'
    return params
